{% extends "base.html" %}

{% block title %}
	&raquo; System &raquo; Chains &raquo; {{paths[0][0][0]}}
{% endblock %}

{% block nav %}
	<a href="/system">System</a> &raquo;
	<a href="/system/chains">Chains</a> &raquo; {{paths[0][0][0]}}
{% endblock %}

{% block content %}
	<h2>Chain</h2>	

	<table>
		<tr>
			<th>Unreachable</th>
			<td>{{unreachable}}</td>
		</tr>
	</table>

	{% for chain in paths %}
		<table>
			<caption>Chain</caption>
			<thead>
					<tr>
						<th>Id</th>
						<th>Type</th>
						<th>Value</th>
					</tr>
			</thead>
			<tbody>
				{% for obj_id, obj_typ, obj_val in chain %}
					<tr>
						<td>{{obj_id}}</td>
						<td>{{obj_typ}}</td>
						<td>{{obj_val}}</td>
					</tr>
				{% endfor %}
			</tbody>
		</table>
	{% endfor %}

{% endblock %}
