{% extends "base.html" %}

{% block title %}
	&raquo; Comms
{% endblock %}

{% block nav %}
	Comms
{% endblock %}

{% block content %}

	<table>
		<thead>
			<tr>
				<th>Code</th>
				<th>Description</th>
			</tr>
		</thead>
		<tbody>
			{% for comm in comms %}
				<tr>
					<td><a href="/e/comms/{{comm.id}}">{{comm.code}}</a></td>
					<td>{{comm.description}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
	
{% endblock %}
