{% extends "base.html" %}

{% block title %}
	DNs
{% endblock %}

{% block nav %}
	DNs
{% endblock %}

{% block content %}
	<table>
		<thead>
			<tr>
				<th>Code</th>
				<th>Name</th>
			</tr>
		</thead>
		<tbody>
			{% for g_dn in g_dns %}
				<tr>
					<td><a href="/g/dns/{{g_dn.id}}">{{g_dn.code}}</a></td>
					<td>{{g_dn.name}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
