{% extends "base.html" %}

{% block title %}
	&raquo; Supplies &raquo;  {{supply.id}} &raquo; Notes
{% endblock %}

{% block nav %}
	<a href="/e/supplies">Supplies</a>  &raquo;
	<a href="/e/supplies/{{supply.id}}">{{supply.id}}</a>  &raquo;
	Notes [<a href="/e/supplies/{{supply.id}}/notes/add">add</a>]
{% endblock %}

{% block content %}
	<table>
		<legend>Notes</legend>
		<thead>
			<tr>
				<th>Edit</th>
				<th>Timestamp</th>
				<th>Body</th>
			</tr>
		</thead>
		<tbody>
			{% for note in supply_note.notes|reverse %}
				<tr>
					<td>
						[<a href="/e/supplies/{{supply.id}}/notes/{{loop.revindex0}}/edit">edit</a>]
					</td>
					<td>{% if note.timestamp %}{{note.timestamp|hh_format}}{% endif %}</td>
					<td>
						<pre style="white-space: pre-wrap;">{{note.body}}</pre>
					</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
