{% extends "base.html" %}

{% block title %}
	Gas Supplies &raquo; {{g_supply.id}} &raquo; Notes &raquo; {{note.index}} Edit
{% endblock %}

{% block nav %}
	<a href="/g_supplies">Gas Supplies</a>  &raquo;
	<a href="/g_supplies/{{g_supply.id}}">{{g_supply.id}}</a>  &raquo;
	<a href="/g_supplies/{{g_supply.id}}/notes">Notes</a>  &raquo;
 	{{note.index}} Edit
{% endblock %}

{% block content %}
	<form method="post" action="/g_supplies/{{g_supply.id}}/notes/{{note.index}}/edit">
		<fieldset>
			<legend>Edit Note</legend>
			<label>Category</label> {{input_text('category', note.category)}}
			<label>Important?</label> {{input_checkbox('is_important', note.is_important)}}
			<label>Body</label> {{input_textarea('body', note.body, 30, 80)}}
			<input type="submit" name="update" value="Edit">
		</fieldset>
	</form>

	<form action="/g_supplies/{{g_supply.id}}/notes/{{note.index}}/edit" method="post">
		<fieldset>
			<legend>Delete Note</legend>
			<input type="submit" name="delete" value="Delete">
		</fieldset>
	</form>
{% endblock %}
