{% extends "base.html" %}

{% block title %}
	&raquo; List of Local Reports
{% endblock %}

{% block nav %}
	List of Local Reports
{% endblock %}

{% block content %}
	<table>
		<caption>Local Reports</caption>	
		<thead>
			<tr>
				<th>View</th>
				<th>Name</th>
			</tr>
		</thead>
		<tbody>
			{% for report in reports %}
				<tr>
					<td><a href="/local_reports/{{report.id}}">View</a></td>
					<td>{{report.name}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>

	<form method="post">
		<fieldset>
			<legend>Add a report</legend>	
			<label>Name</label>
			<input name="name" value="{{request.values['name']}}">
			<input type="submit" value="Add">
		</fieldset>
	</form>
{% endblock %}
