{% extends "base.html" %}

{% block title %}
	&raquo; Non-Core Contracts
{% endblock %}

{% block nav %}
	Non-Core Contracts
{% endblock %}

{% block content %}
	<table>
		<caption>Non-Core Contracts</caption>	
		<thead>
			<tr>
				<th>Name</th>
				<th>Start Date</th>
				<th>Finish Date</th>
			</tr>
		</thead>
		<tbody>
			{% for contract in non_core_contracts %}
				<tr>
					<td><a href="/non_core_contracts/{{contract.id}}">{{contract.name}}</a></td>
					<td>{{ contract.start_rate_script.start_date|hh_format }}</td>
					<td>{{ contract.finish_rate_script.finish_date|hh_format }}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
